library ieee;   
use ieee.std_logic_1164.all;   
use ieee.std_logic_unsigned.all;   
use ieee.std_logic_arith.all;
 
entity raminfr is  
 generic ( bits : integer := 32; 
             -- number of bits per RAM word
            addr_bits : integer := 8); 
             -- 2^addr_bits = number of words in RAM 	
  port (clk  : in std_logic;   
        we   : in std_logic;   
        a    : in std_logic_vector(addr_bits-1 downto 0);   
        dpra : in std_logic_vector(addr_bits-1 downto 0);   
        di   : in std_logic_vector(bits-1 downto 0);   
        spo  : out std_logic_vector(bits-1 downto 0);   
        dpo  : out std_logic_vector(bits-1 downto 0));   
end raminfr;   						   

architecture syn of raminfr is   
  type ram_type is array (2**addr_bits-1 downto 0) of std_logic_vector (bits-1 downto 0);   
  signal RAM : ram_type;   
  signal read_a, read_dpra: std_logic_vector(addr_bits-1 downto 0);
begin   
  process (clk)   
  begin   
    if (clk'event and clk = '1') then   
      if (we = '1') then   
        RAM(conv_integer(unsigned(a))) <= di;  
      end if;   
      read_a <= a;
      read_dpra <= dpra;
    end if;   
  end process;   
  spo <= RAM(conv_integer(unsigned(read_a)));   
  dpo <= RAM(conv_integer(unsigned(read_dpra)));   
 end syn; 